/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <stdlib.h>

#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "action.h"
#include "parser.h"


static void      action_class_init (ActionClass *klass);
static void      action_init       (Action      *action);
static void      action_finalize   (GObject     *object);

static gboolean  action_prepare          (Item         *item);
static void      action_parse_attributes (Item         *item,
                                          const gchar **names,
                                          const gchar **values);


static GObjectClass *parent_class = NULL;


GType
action_get_type (void)
{
  static GType action_type = 0;

  if (!action_type)
    {
      static const GTypeInfo action_info =
      {
        sizeof (ActionClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) action_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (Action),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) action_init,
      };

      action_type = 
        g_type_register_static (TYPE_ITEM, "action", &action_info, 0);
    }
  
  return action_type;
}

static void
action_class_init (ActionClass *klass)
{
  GObjectClass *object_class;
  ItemClass    *item_class;

  object_class = G_OBJECT_CLASS (klass);
  item_class   = ITEM_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  item_class->prepare          = action_prepare;
  item_class->parse_attributes = action_parse_attributes;

  object_class->finalize = action_finalize;
}

static void
action_init (Action *action)
{
  action->key_symbol = DIKS_NULL;
}

Action *
action_new (void)
{
  return ACTION (g_object_new (TYPE_ACTION, NULL));
}

static void
action_finalize (GObject *object)
{
  if (parent_class->finalize)
    parent_class->finalize (object);
}

static gboolean
action_prepare (Item *item)
{
  Action *action;

  action = ACTION (item);

  /*  FIXME: check executable here  */

  return TRUE;
}

static void
action_parse_attributes (Item         *item,
                         const gchar **names,
                         const gchar **values)
{
  const gchar *key;

  key = parse_attributes (names, values, "key");
  
  if (key && *key && *key == 'F')
    {
      gchar *end;
      gint   num;

      num = strtol (key + 1, &end, 0);
      if (end && *end == '\0')
        {
          num += DIKS_F1 - 1;
          if (num >= DIKS_F1 && num <= DIKS_F12)
            ACTION (item)->key_symbol = num;
        }
    }
}
     
void
action_execute (IDirectFB *dfb,
                Action    *action)
{
  gchar *command;
  
  g_return_if_fail (IS_ACTION (action));

  command = ITEM (action)->value;

  g_return_if_fail (command != NULL);

  dfb->Suspend (dfb);
  system (command);
  dfb->Resume (dfb); 
}
