/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "image.h"
#include "resources.h"


static void      image_class_init (ImageClass *klass);
static void      image_finalize   (GObject     *object);
static gboolean  image_prepare    (Item        *item);
static void      image_realize    (Item        *item);
static void      image_unrealize  (Item        *item);


static GObjectClass *parent_class = NULL;


GType
image_get_type (void)
{
  static GType image_type = 0;

  if (!image_type)
    {
      static const GTypeInfo image_info =
      {
        sizeof (ImageClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) image_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (Image),
	0,              /* n_preallocs    */
	NULL,           /* instance_init  */
      };

      image_type = 
        g_type_register_static (TYPE_ITEM, "image", &image_info, 0);
    }
  
  return image_type;
}

static void
image_class_init (ImageClass *klass)
{
  GObjectClass *object_class;
  ItemClass    *item_class;

  object_class = G_OBJECT_CLASS (klass);
  item_class   = ITEM_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  item_class->prepare   = image_prepare;
  item_class->realize   = image_realize;
  item_class->unrealize = image_unrealize;

  object_class->finalize = image_finalize;
}

Image *
image_new (void)
{
  return IMAGE (g_object_new (TYPE_IMAGE, NULL));
}

static void
image_finalize (GObject *object)
{
  if (parent_class->finalize)
    parent_class->finalize (object);
}

static gboolean
image_prepare (Item *item)
{
  return (resources_get_image_provider (item->value) != NULL);
}

static void
image_realize (Item *item)
{
  resources_get_surface (item->value);
}

static void
image_unrealize (Item *item)
{
  resources_release_surface (item->value);
}
