/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#ifndef __ITEM_H__
#define __ITEM_H__

typedef enum
{
  FGCOLOR  = 1 << 0,
  BGCOLOR  = 1 << 1,
  FACE     = 1 << 2,
  BGIMAGE  = 1 << 3,
  BULLET   = 1 << 4,
  SIZE     = 1 << 5,
  WIDTH    = 1 << 6,
  HEIGHT   = 1 << 7,
  XOFFSET  = 1 << 8,
  YOFFSET  = 1 << 9,
  EFFECT   = 1 << 10
} ItemFlags;

enum _EffectFlags
{
  EFFECT_NONE     = 0,
  EFFECT_BLEND    = 1 << 0,
  EFFECT_SLIDE    = 1 << 1,

  EFFECT_LEFT     = 1 << 2,
  EFFECT_RIGHT    = 1 << 3,
  EFFECT_TOP      = 1 << 4,
  EFFECT_BOTTOM   = 1 << 5,

  EFFECT_BACKWARD = 1 << 6
};


#define TYPE_ITEM            (item_get_type ())
#define ITEM(obj)            (G_TYPE_CHECK_INSTANCE_CAST ((obj), TYPE_ITEM, Item))
#define IS_ITEM(obj)         (G_TYPE_CHECK_INSTANCE_TYPE ((obj), TYPE_ITEM))
#define ITEM_CLASS(klass)    (G_TYPE_CHECK_CLASS_CAST ((klass), TYPE_ITEM, ItemClass))
#define IS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), TYPE_ITEM))


typedef struct _ItemClass  ItemClass;

struct _Item
{
  GObject     parent_instance;

  gchar      *value;

  guchar      fgcolor[4];
  guchar      bgcolor[4];
  gchar      *face;
  gchar      *bgimage;
  gchar      *bullet;
  gint        size;
  gint        x;
  gint        y;
  gint        width;
  gint        height;
  EffectFlags effect;
  ItemFlags   flags;  
};

struct _ItemClass
{
  GObjectClass parent_class;

  gboolean (* prepare)          (Item         *item);
  void     (* realize)          (Item         *item);
  void     (* unrealize)        (Item         *item);
  void     (* parse_attributes) (Item         *item,
                                 const gchar **names,
                                 const gchar **values);
};


GType      item_get_type         (void);
Item     * item_new              (const gchar  *name);
gboolean   item_prepare          (Item         *item);
void       item_realize          (Item         *item);
void       item_unrealize        (Item         *item);
void       item_set_value        (Item         *item,
                                  const gchar  *value);
void       item_parse_attributes (Item         *item,
                                  const gchar **names,
                                  const gchar **values);


#endif
