/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "slide.h"


static void      slide_class_init (SlideClass *klass);
static void      slide_init       (Slide      *slide);
static void      slide_finalize   (GObject    *object);
static gboolean  slide_prepare    (Item       *item);
static void      slide_realize    (Item       *item);
static void      slide_unrealize  (Item       *item);


static GObjectClass *parent_class = NULL;


GType
slide_get_type (void)
{
  static GType slide_type = 0;

  if (!slide_type)
    {
      static const GTypeInfo slide_info =
      {
        sizeof (SlideClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) slide_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (Slide),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) slide_init,
      };

      slide_type = 
        g_type_register_static (TYPE_ITEM, "slide", &slide_info, 0);
    }
  
  return slide_type;
}

static void
slide_class_init (SlideClass *klass)
{
  GObjectClass *object_class;
  ItemClass    *item_class;

  object_class = G_OBJECT_CLASS (klass);
  item_class   = ITEM_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  item_class->prepare   = slide_prepare;
  item_class->realize   = slide_realize;
  item_class->unrealize = slide_unrealize;

  object_class->finalize = slide_finalize;
}

static void
slide_init (Slide *slide)
{
  slide->items = NULL;
}

static void
slide_finalize (GObject *object)
{
  g_list_foreach (SLIDE (object)->items, (GFunc) g_object_unref, NULL);

  if (parent_class->finalize)
    parent_class->finalize (object);
}

static gboolean
slide_prepare (Item *item)
{
  GList *list;

  for (list = SLIDE (item)->items; list; list = g_list_next (list))
    {
      if (!item_prepare ((Item *) list->data))
        break;
    }
  return (list == NULL);
}

static void
slide_realize (Item *item)
{
  g_list_foreach (SLIDE (item)->items, (GFunc) item_realize, NULL);
}

static void
slide_unrealize (Item *item)
{
  g_list_foreach (SLIDE (item)->items, (GFunc) item_unrealize, NULL);
}

void
slide_append_item (Slide *slide,
                   Item  *item)
{
  g_return_if_fail (IS_SLIDE (slide));
  g_return_if_fail (IS_ITEM (item));

  slide->items = g_list_append (slide->items, item);
}
