/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <glib-object.h>

#include "apptypes.h"

#include "parser.h"
#include "slide.h"
#include "slides.h"


static void      slides_class_init (SlidesClass *klass);
static void      slides_init       (Slides      *slides);
static void      slides_finalize   (GObject     *object);
static gboolean  slides_prepare    (Item        *item);

static void      slides_parse_attributes (Item         *item,
                                          const gchar **names,
                                          const gchar **values);


static GObjectClass *parent_class = NULL;


GType
slides_get_type (void)
{
  static GType slides_type = 0;

  if (!slides_type)
    {
      static const GTypeInfo slides_info =
      {
        sizeof (SlidesClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) slides_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (Slides),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) slides_init,
      };

      slides_type = 
        g_type_register_static (TYPE_ITEM, "slides", &slides_info, 0);
    }
  
  return slides_type;
}

static void
slides_class_init (SlidesClass *klass)
{
  GObjectClass *object_class;
  ItemClass    *item_class;

  object_class = G_OBJECT_CLASS (klass);
  item_class   = ITEM_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  item_class->prepare          = slides_prepare;
  item_class->parse_attributes = slides_parse_attributes;

  object_class->finalize = slides_finalize;
}

static void
slides_init (Slides *slides)
{
  slides->slides        = NULL;
  slides->screen_width  = 0;
  slides->screen_height = 0;
  slides->bpp           = 16;
}

static void
slides_finalize (GObject *object)
{
  g_list_foreach (SLIDES (object)->slides, (GFunc) g_object_unref, NULL);

  if (parent_class->finalize)
    parent_class->finalize (object);
}

static gboolean
slides_prepare (Item *item)
{
  GList *list;

  for (list = SLIDES (item)->slides; list; list = g_list_next (list))
    {
      if (!item_prepare ((Item *) list->data))
        break;
    }

  return (list == NULL);
}

void
slides_append_slide (Slides *slides,
                     Slide  *slide)
{
  g_return_if_fail (IS_SLIDES (slides));
  g_return_if_fail (IS_SLIDE (slide));

  slides->slides = g_list_append (slides->slides, slide);
}

static void
slides_parse_attributes (Item         *item,
                         const gchar **names,
                         const gchar **values)
{
  Slides      *slides;
  const gchar *num;

  slides = SLIDES (item);

  num = parse_attributes (names, values, "screen_width");
  parse_number (num, &slides->screen_width);

  num = parse_attributes (names, values, "screen_height");
  parse_number (num, &slides->screen_height);

  num = parse_attributes (names, values, "bpp");
  parse_number (num, &slides->bpp);
}
