/* DirectFB presentation app
 *
 * Copyright (C) 2001  convergence integrated media
 * Authors: Sven Neumann <sven@convergence.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */


#include <directfb.h>
#include <glib-object.h>

#include "apptypes.h"

#include "resources.h"
#include "video.h"


static void      video_class_init (VideoClass   *klass);
static void      video_init       (Video        *video);
static void      video_finalize   (GObject      *object);
static gboolean  video_prepare    (Item         *item);
static gint      video_callback   (gpointer      ctxt);


static GObjectClass *parent_class = NULL;


GType
video_get_type (void)
{
  static GType video_type = 0;

  if (!video_type)
    {
      static const GTypeInfo video_info =
      {
        sizeof (VideoClass),
	(GBaseInitFunc) NULL,
	(GBaseFinalizeFunc) NULL,
	(GClassInitFunc) video_class_init,
	NULL,		/* class_finalize */
	NULL,		/* class_data     */
	sizeof (Video),
	0,              /* n_preallocs    */
	(GInstanceInitFunc) video_init
      };

      video_type = 
        g_type_register_static (TYPE_IMAGE, "video", &video_info, 0);
    }
  
  return video_type;
}

static void
video_class_init (VideoClass *klass)
{
  GObjectClass *object_class;
  ItemClass    *item_class;

  object_class = G_OBJECT_CLASS (klass);
  item_class   = ITEM_CLASS (klass);
  parent_class = g_type_class_peek_parent (klass);

  item_class->prepare = video_prepare;

  object_class->finalize = video_finalize;
}

static void
video_init (Video *video)
{
  video->dest_surface = NULL;
}

static void
video_finalize (GObject *object)
{
  if (parent_class->finalize)
    parent_class->finalize (object);
}

static gboolean
video_prepare (Item *item)
{
  return (resources_get_video_provider (item->value) != NULL);
}

void
video_play (Video *video)
{
  Item                   *item;
  IDirectFBVideoProvider *provider; 
  DFBSurfaceDescription   dsc;
  DFBRectangle            rect;

  g_return_if_fail (video->dest_surface != NULL);

  provider = resources_get_video_provider (ITEM (video)->value);
  provider->GetSurfaceDescription (provider, &dsc);

  item = (Item *) video;
  
  rect.w = item->flags & WIDTH   ? item->width  : dsc.width;
  rect.h = item->flags & HEIGHT  ? item->height : dsc.height;
  rect.x = item->flags & XOFFSET ? item->x      : 0;
  rect.y = item->flags & YOFFSET ? item->y      : 0;

  provider->PlayTo (provider, video->dest_surface, &rect, 
                    video_callback, video->dest_surface);
}

void
video_stop (Video *video)
{
  IDirectFBVideoProvider *provider; 

  provider = resources_get_video_provider (ITEM (video)->value);

  provider->Stop (provider);
}

static gint
video_callback (gpointer ctxt)
{
  IDirectFBSurface *surface = ctxt;

  /*  FIXME: should only flip the relevant part of the window 
   *         or use a subsurface for the video area  
   */

  surface->Flip (surface, NULL, 0);

  return 0;
}
